'
' DrawMenu.bas - sample of how to draw bitmaps in ownerdrawn menus.
' Public Domain by Borje Hagsten, October, 2000.
' Free to use and modify, but as always, at own responsibility.
'
' To understand flow. A menu is created and attached to dialog in SUB addMenu.
' Each item gets text and eventual bitmap from FUNCTION GetMenuTxtAndBmp.
' Windows does not provide enough space for a bitmap by default, so we need to
' indent text a bit in ownerdrawn items - therefore I can recommend to make all
' items ownerdrawn - even those without bitmap - to get consistent alignment.
'
' For use in other code - copy code under %WM_DRAWITEM, %WM_MEASUREITEM and
' %WM_SYSCOLORCHANGE, along with related routines (DrawMenu, MeasureMenu).
' Then setup text and bitmap numbers relateed to item id's in GetMenuTxtAndBmp
' and make sure all items are made ownerdrawn in addMenu routine.

' LOG
' May 17, 2003: Major rewrite of several routines.
' Feb 13, 2001: Changed a few DST_ values for DrawState in DrawMenu to DWORDS.
' Feb 12, 2001:
'  Complete change of way to store menu item text. Now use a separate function
'  for this, GetMenuTxtAndBmp. Advantages - is cleaner and no more hassle with
'  GetMenuItemInfo, which seems to behave differently depending on system.
'  Win95/98, different versions of NT 4 and 5 - they all seem to behave
'  differently. Think MS has managed to screw up real bad there..
'
' Files: DrawMenu.exe, DrawMenu.bas, DrawMenu.pbr, DrawMenu.rc and Toolbar.bmp
' The bmp contains some 16x16 bitmaps for the menu items. Drawmenu.zip can be
' downloaded from : http://www.tolkenxp.com/pb/drawmenu.zip
'
' Many thanks to Fred Oxenby, Semen Matusovski, Paul Noble and
' Paul Squires for valuable tips and help with/during testing.
'
#COMPILE EXE
#RESOURCE "DRAWMENU.PBR"
#INCLUDE  "WIN32API.INC"
'--------------------------------------------------------------------
%IDM_FILE    = %WM_USER + 200
%IDM_NEW     = %WM_USER + 201
%IDM_OPEN    = %WM_USER + 202
%IDM_SAVE    = %WM_USER + 203
%IDM_SAVEAS  = %WM_USER + 204
%IDM_EDIT    = %WM_USER + 220
%IDM_UNDO    = %WM_USER + 221
%IDM_CUT     = %WM_USER + 223
%IDM_COPY    = %WM_USER + 224
%IDM_PASTE   = %WM_USER + 225
%IDM_SELALL  = %WM_USER + 226
'--------------------------------------------------------------------
DECLARE CALLBACK FUNCTION DlgProc
 
DECLARE SUB addMenu     (BYVAL hDlg AS DWORD)
DECLARE SUB DrawMenu    (BYREF lp AS LONG)
DECLARE SUB MeasureMenu (BYVAL hDlg AS DWORD, BYVAL lParam AS DWORD)
 
DECLARE FUNCTION GetMenuTxtAndBmp (BYVAL ItemId AS LONG, BmpNum AS LONG) AS STRING
DECLARE FUNCTION GetMenuBmpHandle (BYVAL bNum AS LONG) AS LONG
  
'
' Program entrance
'--------------------------------------------------------------------
FUNCTION PBMAIN
  LOCAL hDlg AS DWORD
 
  DIALOG NEW 0, "Bmp Menu Demo", , , 200, 120, %WS_CAPTION OR %WS_SYSMENU, TO hDlg
 
  addMenu hDlg 'add a menu to the dialog in separate routine
 
  DIALOG SHOW MODAL hDlg CALL DlgProc
END FUNCTION
 
'
' Main Dialog callback
'--------------------------------------------------------------------
CALLBACK FUNCTION DlgProc
  LOCAL hMnu AS DWORD

  SELECT CASE CBMSG
     CASE %WM_COMMAND
        SELECT CASE CBCTL
           CASE %IDCANCEL   ' note: also triggered with Escape key
              IF CBCTLMSG = %BN_CLICKED OR CBCTLMSG = 1 THEN
                 DIALOG END CBHNDL ' End the program
              END IF
        END SELECT
 
     CASE %WM_DRAWITEM
        IF CBWPARAM = 0 THEN  ' if identifier is 0, message was sent by a menu
           DrawMenu CBLPARAM  ' draw the menu
           FUNCTION = %TRUE : EXIT FUNCTION
        END IF

     CASE %WM_MEASUREITEM      ' get menu item size
        IF CBWPARAM = 0 THEN   ' a menu is calling
           MeasureMenu CBHNDL, CBLPARAM   ' do all work in separate Sub
           FUNCTION = %TRUE : EXIT FUNCTION
        END IF

     CASE %WM_SYSCOLORCHANGE    ' if user changes color scheme,
        hMnu = GetMenu(CBHNDL)  ' menu font size may have changed,
        IF hMnu THEN DestroyMenu hMnu  ' so destroy old menu and create a
        addMenu CBHNDL                 ' new, to use new font's measurements

  END SELECT
 
END FUNCTION
 
'
' Add a menu to given window/dialog
'--------------------------------------------------------------------
SUB addMenu (BYVAL hDlg AS DWORD)
  LOCAL hMnu AS DWORD, hmnuFile AS DWORD, hmnuEdit AS DWORD

  MENU NEW BAR TO hMnu
  MENU NEW POPUP TO hmnuFile
     MENU ADD POPUP, hMnu, GetMenuTxtAndBmp(%IDM_FILE, 0), hmnuFile, %MF_ENABLED
     MENU ADD STRING, hMnuFile, GetMenuTxtAndBmp(%IDM_NEW, 0),    %IDM_NEW,    %MF_ENABLED
     MENU ADD STRING, hMnuFile, GetMenuTxtAndBmp(%IDM_OPEN, 0),   %IDM_OPEN,   %MF_ENABLED
     MENU ADD STRING, hMnuFile, "-", 0, 0
     MENU ADD STRING, hMnuFile, GetMenuTxtAndBmp(%IDM_SAVE, 0),   %IDM_SAVE,   %MF_ENABLED
     MENU ADD STRING, hMnuFile, GetMenuTxtAndBmp(%IDM_SAVEAS, 0), %IDM_SAVEAS, %MF_ENABLED
     MENU ADD STRING, hMnuFile, "-", 0, 0
     MENU ADD STRING, hMnuFile, GetMenuTxtAndBmp(%IDCANCEL, 0),   %IDCANCEL,   %MF_ENABLED
  MENU NEW POPUP TO hmnuEdit
     MENU ADD POPUP, hMnu, GetMenuTxtAndBmp(%IDM_EDIT, 0), hmnuEdit, %MF_ENABLED
     MENU ADD STRING, hmnuEdit, GetMenuTxtAndBmp(%IDM_UNDO, 0),   %IDM_UNDO,   %MF_ENABLED
     MENU ADD STRING, hMnuEdit, "-", 0, 0
     MENU ADD STRING, hmnuEdit, GetMenuTxtAndBmp(%IDM_CUT, 0),    %IDM_CUT,    %MF_ENABLED
     MENU ADD STRING, hmnuEdit, GetMenuTxtAndBmp(%IDM_COPY, 0),   %IDM_COPY,   %MF_ENABLED
     MENU ADD STRING, hmnuEdit, GetMenuTxtAndBmp(%IDM_PASTE, 0),  %IDM_PASTE,  %MF_ENABLED
     MENU ADD STRING, hMnuEdit, "-", 0, 0
     MENU ADD STRING, hmnuEdit, GetMenuTxtAndBmp(%IDM_SELALL, 0), %IDM_SELALL, %MF_ENABLED
  MENU ATTACH hMnu, hDlg

  'Make all menu items ownerdrawn
  ModifyMenu hmnuFile, %IDM_NEW,    %MF_OWNERDRAW, %IDM_NEW,    BYVAL %NULL
  ModifyMenu hmnuFile, %IDM_OPEN,   %MF_OWNERDRAW, %IDM_OPEN,   BYVAL %NULL
  ModifyMenu hmnuFile, %IDM_SAVE,   %MF_OWNERDRAW, %IDM_SAVE,   BYVAL %NULL
  ModifyMenu hmnuFile, %IDM_SAVEAS, %MF_OWNERDRAW, %IDM_SAVEAS, BYVAL %NULL
  ModifyMenu hmnuFile, %IDCANCEL,   %MF_OWNERDRAW, %IDCANCEL,   BYVAL %NULL
 
  ModifyMenu hmnuEdit, %IDM_UNDO,   %MF_OWNERDRAW, %IDM_UNDO,   BYVAL %NULL
  ModifyMenu hmnuEdit, %IDM_CUT,    %MF_OWNERDRAW, %IDM_CUT,    BYVAL %NULL
  ModifyMenu hmnuEdit, %IDM_COPY,   %MF_OWNERDRAW, %IDM_COPY,   BYVAL %NULL
  ModifyMenu hmnuEdit, %IDM_PASTE,  %MF_OWNERDRAW, %IDM_PASTE,  BYVAL %NULL
  ModifyMenu hmnuEdit, %IDM_SELALL, %MF_OWNERDRAW, %IDM_SELALL, BYVAL %NULL

END SUB

'
' Return menu item string and one-based bitmap number (0 = no bitmap)
'--------------------------------------------------------------------
FUNCTION GetMenuTxtAndBmp (BYVAL ItemId AS LONG, BmpNum AS LONG) AS STRING
  SELECT CASE ItemId
     CASE %IDM_FILE    : BmpNum = 0 : FUNCTION = "&File"
     CASE %IDM_NEW     : BmpNum = 1 : FUNCTION = "&New"     & $TAB & "Ctrl + N"
     CASE %IDM_OPEN    : BmpNum = 2 : FUNCTION = "&Open..." & $TAB & "Ctrl + O"
     CASE %IDM_SAVE    : BmpNum = 3 : FUNCTION = "&Save"    & $TAB & "Ctrl + S"
     CASE %IDM_SAVEAS  : BmpNum = 0 : FUNCTION = "&Save as..."
     CASE %IDCANCEL    : BmpNum = 8 : FUNCTION = "E&xit"    & $TAB & "Alt + F4"
 
     CASE %IDM_EDIT    : BmpNum = 0 : FUNCTION = "&Edit"
     CASE %IDM_UNDO    : BmpNum = 7 : FUNCTION = "&Undo"    & $TAB & "Ctrl + Z"
     CASE %IDM_CUT     : BmpNum = 4 : FUNCTION = "Cu&t"     & $TAB & "Ctrl + X"
     CASE %IDM_COPY    : BmpNum = 5 : FUNCTION = "&Copy"    & $TAB & "Ctrl + C"
     CASE %IDM_PASTE   : BmpNum = 6 : FUNCTION = "&Paste"   & $TAB & "Ctrl + V"
     CASE %IDM_SELALL  : BmpNum = 0 : FUNCTION = "Select &All" & $TAB & "Ctrl + A"
  END SELECT
END FUNCTION

'
' Draw ownerdrawn menu items - called from %WM_DRAWITEM
'--------------------------------------------------------------------
SUB DrawMenu (BYREF lp AS LONG)
  LOCAL c                AS LONG        ' for/next counter
  LOCAL ShortCutWidth    AS LONG        ' for ShortCut text width (if any)
  LOCAL id               AS LONG        ' tmp item id
  LOCAL BmpNum           AS LONG        ' bitmap number
  LOCAL hBmp             AS LONG        ' icon to draw
  LOCAL hBrush           AS LONG        ' brush used for selection draw
  LOCAL bHighlight       AS LONG        ' higlighted flag
  LOCAL drawTxtFlags     AS DWORD       ' how we draw the text
  LOCAL drawBmpFlags     AS DWORD       ' how we draw the icon
  LOCAL menuBtnW         AS LONG        ' Width of the menubutton, in pixels
  LOCAL sCaption         AS STRING      ' string for the menu item text
  LOCAL sShortCut        AS STRING      ' string for eventual shortcut text
  LOCAL rc               AS RECT        ' RECT used for drawing areas
  LOCAL szl              AS SIZEL       ' used to calculate text extent
  LOCAL lpDis AS DRAWITEMSTRUCT PTR     ' we fill this from received pointer
 
  lpDis    = lp ' Get structure from given pointer
  menuBtnW = GetSystemMetrics(%SM_CYMENU) - 1 ' for "button" size

  '------------------------------------------------------------------
  ' Calculate pos for eventual shortcut. We need to look at all items
  ' in dropped menu and calculate largest width, which then is used
  ' as base to calculate all shortcuts' pos.
  '------------------------------------------------------------------
  FOR c = 0 TO GetMenuItemCount(@lpDis.hwndItem) - 1 'hwndItem is dropped menu's handle
     id = GetMenuItemID(@lpDis.hwndItem, c)

     IF id THEN
        sCaption = GetMenuTxtAndBmp(id, 0)
        IF INSTR(sCaption, $TAB) THEN       'if it has shortcut (Ctrl+X, etc.)
           sShortCut = TRIM$(PARSE$(sCaption, $TAB, 2))
        ELSE
           sShortCut = ""
        END IF

        IF LEN(sShortCut) THEN
           GetTextExtentPoint32 @lpDis.hDC, BYVAL STRPTR(sShortCut), LEN(sShortCut), BYVAL VARPTR(szl)
           ShortCutWidth = MAX&(ShortCutWidth, szl.cx)
        END IF
     END IF
  NEXT

  '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  ' Get menu item string and split up into ev. text and shortcut part
  '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  sCaption = GetMenuTxtAndBmp(@lpDis.itemID, BmpNum)
  IF INSTR(sCaption, $TAB) THEN       'if it has shortcut (Ctrl+X, etc.)
     sShortCut = TRIM$(PARSE$(sCaption, $TAB, 2))
     sCaption  = TRIM$(PARSE$(sCaption, $TAB, 1))
  ELSE
     sCaption = TRIM$(sCaption)
     sShortCut = ""
  END IF
 
  '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  ' Calculate menu item string's height
  '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  GetTextExtentPoint32 @lpDis.hDC, BYVAL STRPTR(sCaption), LEN(sCaption), szl
 
  '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  ' Sep up colors depending on what drawing actions we need to take
  '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  IF (@lpDis.itemState AND %ODS_SELECTED) THEN                     'menu item is selected
     hBrush =     GetSysColorBrush(%COLOR_HIGHLIGHT)
     SetBkColor   @lpDis.hDC, GetSysColor(%COLOR_HIGHLIGHT)
     SetTextColor @lpDis.hDC, GetSysColor(%COLOR_HIGHLIGHTTEXT)
     bHighlight   = %TRUE  'flag, so later we can add or remove the 3D edge around the icon
  ELSEIF (@lpDis.itemState AND %ODS_GRAYED) THEN                   'menu item is disabled
     hBrush =     GetSysColorBrush(%COLOR_MENU)
     SetBkColor   @lpDis.hDC, GetSysColor(%COLOR_MENU)
     SetTextColor @lpDis.hDC, GetSysColor(%COLOR_MENUTEXT)
  ELSE                                                            'not selected or disabled
     hBrush =     GetSysColorBrush(%COLOR_MENU)
     SetBkColor   @lpDis.hDC, GetSysColor(%COLOR_MENU)
     SetTextColor @lpDis.hDC, GetSysColor(%COLOR_MENUTEXT)
  END IF
 
  drawTxtFlags = %DST_PREFIXTEXT
  drawBmpFlags = %DST_BITMAP
 
  IF (@lpDis.itemState AND %ODS_GRAYED) THEN      'if disabled item
     IF (@lpDis.itemState AND %ODS_SELECTED) THEN 'if it's selected
        SetTextColor @lpDis.hDC, GetSysColor(%COLOR_GRAYTEXT)
     ELSE
        drawTxtFlags = drawTxtFlags OR %DSS_DISABLED
     END IF
  END IF
 
  '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  ' Calculate rect for highlight area (selected item)
  ' and fill rect with proper color, either COLOR_MENU
  ' or COLOR_HIGHLIGHT, depending on selection state.
  '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  rc = @lpDis.rcItem
  IF (@lpDis.itemState AND %ODS_GRAYED) OR BmpNum = 0 THEN 'grayed or no bitmap
     rc.nLeft = 0
  ELSE
     rc.nLeft = menuBtnW + 4  'enabled, with bitmap
  END IF
  FillRect @lpDis.hDC, rc, hBrush
 
  '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  ' Draw the bitmap "button", if item is not grayed out (disabled)
  '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  IF (@lpDis.itemState AND %ODS_GRAYED) = 0 THEN
     IF BmpNum > 0 THEN hBmp = GetMenuBmpHandle(BmpNum - 1)
 
     IF hBmp <> 0 THEN     'Draw bitmap, centered in "button"
        DrawState @lpDis.hDC, 0&, 0&, hBmp, 0&, ((menuBtnW + 3) \ 2) - 8, _
                  ((@lpDis.rcItem.nTop + @lpDis.rcItem.nBottom) \ 2) - 8, _
                  16, 16, drawBmpFlags
        DeleteObject hBmp 'Delete the bitmap when done, to avoid memory leaks!
 
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        ' Calculate the RECT we need for the 3D edge
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        rc = @lpDis.rcItem
        rc.nLeft   = 0               'Size and pos for "button"..
        rc.nRight  = menuBtnW + 3
        rc.nBottom = rc.nTop  + GetSystemMetrics(%SM_CYMENU) + 1
 
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        ' Draw "button" if the item is highlighted, or flat if it's not
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        IF bHighlight = %TRUE THEN
           DrawEdge @lpDis.hDC, rc, %BDR_RAISEDINNER, %BF_RECT
        ELSE
           DrawEdge @lpDis.hDC, rc, %BDR_RAISEDINNER, %BF_FLAT OR %BF_RECT
        END IF
     END IF
  END IF
 
  '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  ' Draw the menu text next to bitmap button, with centered y-pos
  '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  IF LEN(sCaption) THEN
       rc.nLeft = menuBtnW + 6 : rc.nRight = @lpDis.rcItem.nRight
       rc.nTop = rc.nTop + ((rc.nBottom - rc.nTop) - szl.cy) \ 2
       DrawState @lpDis.hDC, 0&, 0&, BYVAL STRPTR(sCaption), LEN(sCaption), _
                 rc.nLeft, rc.nTop, rc.nRight, szl.cy, drawTxtFlags

     IF ShortCutWidth THEN 'if there's shortcut text (Ctrl+N, etc)
        DrawState @lpDis.hDC, 0&, 0&, BYVAL STRPTR(sShortCut), LEN(sShortCut), _
                  @lpDis.rcItem.nRight - ShortCutWidth - menuBtnW - 6, rc.nTop, _
                  rc.nRight, szl.cy, drawTxtFlags
     END IF
  END IF
 
END SUB

'
' GetMenuBmpHandle - get desired menu item's bitmap handle
'--------------------------------------------------------------------
FUNCTION GetMenuBmpHandle (BYVAL BmpNum AS LONG) AS LONG
  LOCAL hBmp1 AS DWORD, hBmp2 AS DWORD, _
        hDC AS DWORD, memDC1  AS DWORD, memDC2 AS DWORD
 
  hBmp1 = LoadImage(GetModuleHandle(""), BYVAL 1, _  'Load Bmp strip from resource
                    %IMAGE_BITMAP, 0, 0, %LR_LOADMAP3DCOLORS)

  hdc = GetDc(0)                     ' Get device context for screen
  memDC1 = CreateCompatibleDC (hDC)  ' Create two memory device contexts
  memDC2 = CreateCompatibleDC (hDC)
  hBmp2  = CreateCompatibleBitmap(hDC, 16, 16) ' Create compatible 16x16 pixel bmp
  ReleaseDC 0, hdc                   ' Release the DC - don't need it anymore.

  SelectObject memDC1, hBmp1         ' Select the bitmaps into the mem dc's
  SelectObject memDC2, hBmp2
 
  BitBlt memDC2, 0, 0, 16, 16, _     ' copy bNum part of hBmp1 strip to hBmp2
         memDC1, BmpNum * 16, 0, %SRCCOPY
 
  DeleteDC memDC1       'Cleanup memDC's and delete main bmp strip object
  DeleteDC memDC2
  DeleteObject hBmp1    '(note: 16x16 pixel hBmp2 is deleted in DrawMenu routine)
 
  FUNCTION = hBmp2      'return handle to 16x16 pixel bitmap
END FUNCTION

'
' MeasureMenu - %WM_MEASUREITEM handler
'--------------------------------------------------------------------
SUB MeasureMenu (BYVAL hDlg AS DWORD, BYVAL lParam AS DWORD)
 LOCAL hDC AS DWORD, hf AS DWORD, txt AS STRING, sl AS SIZEL
 LOCAL lpMis AS MEASUREITEMSTRUCT PTR, ncm AS NONCLIENTMETRICS

  lpMis = lParam     'lParam points to a MEASUREITEMSTRUCT structure
  txt   = GetMenuTxtAndBmp(@lpMis.itemID, 0) 'get menu item's text

  hDC = GetDC(hDlg)          'grab dialog's DC
    ncm.cbSize = SIZEOF(ncm) 'grab menu font
    SystemParametersInfo %SPI_GETNONCLIENTMETRICS, SIZEOF(ncm), BYVAL VARPTR(ncm), 0
    IF LEN(ncm.lfMenuFont) THEN
       hf = CreateFontIndirect(ncm.lfMenuFont) 'create font from menu LogFont data
       IF hf THEN hf = SelectObject(hDC, hf)   'select it into DC
    END IF
    GetTextExtentPoint32 hDC, BYVAL STRPTR(txt), LEN(txt), sl 'get text size
    @lpMis.itemWidth  = sl.cx                            'set width
    @lpMis.ItemHeight = GetSystemMetrics(%SM_CYMENU) + 1 'set height
    IF hf THEN DeleteObject SelectObject(hDC, hf)        'delete our tmp menu font object
  ReleaseDC hDlg, hDC  'release DC to avoid memory leak

END SUB
